@@ -14,7 +14,7 @@ package main
 import (
 	"bytes"
 	"encoding/binary"
-	"fmt"
+	"errors"
 	"log"
 	"net"
 	"sync"
@@ -73,11 +73,11 @@ func scan(iface *net.Interface) error {
 	}
 	// Sanity-check that the interface has a good address.
 	if addr == nil {
-		return fmt.Errorf("no good IP network found")
+		return errors.New("no good IP network found")
 	} else if addr.IP[0] == 127 {
-		return fmt.Errorf("skipping localhost")
+		return errors.New("skipping localhost")
 	} else if addr.Mask[0] != 0xff || addr.Mask[1] != 0xff {
-		return fmt.Errorf("mask means network is too large")
+		return errors.New("mask means network is too large")
 	}
 	log.Printf("Using network range %v for interface %v", addr, iface.Name)
 
@@ -181,8 +181,8 @@ func ips(n *net.IPNet) (out []net.IP) {
 		var buf [4]byte
 		binary.BigEndian.PutUint32(buf[:], num)
 		out = append(out, net.IP(buf[:]))
-		mask += 1
-		num += 1
+		mask++
+		num++
 	}
 	return
 }
