@@ -118,7 +118,7 @@ func (s *myStream) Reassembled(rs []tcpassembly.Reassembly) {
 		// Mark that we've received new packet data.
 		// We could just use time.Now, but by using r.Seen we handle the case
 		// where packets are being read from a file and could be very old.
-		if s.bidi.lastPacketSeen.After(r.Seen) {
+		if s.bidi.lastPacketSeen.Before(r.Seen) {
 			s.bidi.lastPacketSeen = r.Seen
 		}
 	}
@@ -163,6 +163,10 @@ func main() {
 	streamFactory := &myFactory{bidiMap: make(map[key]*bidi)}
 	streamPool := tcpassembly.NewStreamPool(streamFactory)
 	assembler := tcpassembly.NewAssembler(streamPool)
+	// Limit memory usage by auto-flushing connection state if we get over 100K
+	// packets in memory, or over 1000 for a single stream.
+	assembler.MaxBufferedPagesTotal = 100000
+	assembler.MaxBufferedPagesPerConnection = 1000
 
 	log.Println("reading in packets")
 	// Read in packets, pass to assembler.
