@@ -78,6 +78,125 @@ func TestParseDNSTypeTXT(t *testing.T) {
 	}
 }
 
+var testParseDNSBadVers = []byte{
+	0x02, 0x00, 0x00, 0x00, // PF_INET
+	0x45, 0x00, 0x00, 0x38, 0xa5, 0xa0, 0x40, 0x00, 0x38, 0x11, 0x00, 0xbd, 0xc0, 0x05, 0x05, 0xf1,
+	0xac, 0x1e, 0x2a, 0x43, 0x00, 0x35, 0xfd, 0x78, 0x00, 0x24, 0x40, 0xc1, 0x8f, 0xb3, 0x81, 0x00,
+	0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x10, 0x00, 0x01, 0x00, 0x00, 0x29,
+	0x02, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
+}
+var testParseDNSBadVersResponseCode = DNSResponseCodeBadVers
+
+func TestParseDNSBadVers(t *testing.T) {
+	p := gopacket.NewPacket(testParseDNSBadVers, LinkTypeNull, testDecodeOptions)
+	if p.ErrorLayer() != nil {
+		t.Error("Failed to decode packet:", p.ErrorLayer().Error())
+	}
+	checkLayers(p, []gopacket.LayerType{LayerTypeLoopback, LayerTypeIPv4, LayerTypeUDP, LayerTypeDNS}, t)
+	questions := p.Layer(LayerTypeDNS).(*DNS).Questions
+	if len(questions) != 1 {
+		t.Error("Failed to parse 1 DNS question")
+	}
+	answers := p.Layer(LayerTypeDNS).(*DNS).Answers
+	if len(answers) != 0 {
+		t.Error("Failed to parse 0 DNS answer")
+	}
+	additionals := p.Layer(LayerTypeDNS).(*DNS).Additionals
+	if len(additionals) != 1 {
+		t.Error("Failed to parse 1 DNS additional")
+	}
+
+	optAll := additionals[0].OPT
+	if len(optAll) != 0 {
+		t.Errorf("Parsed %d OPTs, expected 0", len(optAll))
+	}
+	responseCode := p.Layer(LayerTypeDNS).(*DNS).ResponseCode
+	if responseCode != testParseDNSBadVersResponseCode {
+		t.Errorf("Incorrect extended response code, expected %q, got %q", testParseDNSBadVersResponseCode, responseCode)
+	}
+}
+
+var testParseDNSBadCookie = []byte{
+	0x02, 0x00, 0x00, 0x00, // PF_INET
+	0x45, 0x00, 0x00, 0x54, 0xfe, 0xaa, 0x00, 0x00, 0x40, 0x11, 0x00, 0x00, 0x7f, 0x00, 0x00, 0x01,
+	0x7f, 0x00, 0x00, 0x01, 0x00, 0x35, 0xd6, 0xaa, 0x00, 0x40, 0xfe, 0x53, 0xf6, 0xab, 0x81, 0x87,
+	0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x06, 0x00, 0x01, 0x00, 0x00, 0x29,
+	0x10, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x1c, 0x00, 0x0a, 0x00, 0x18, 0x36, 0xbf, 0x11, 0x1f,
+	0xef, 0x2e, 0x01, 0x09, 0x7d, 0x8f, 0xfe, 0x06, 0x5c, 0x63, 0x6f, 0xfb, 0x14, 0x2d, 0x76, 0x74,
+	0x94, 0x40, 0x7a, 0x73,
+}
+var testParseDNSBadCookieResponseCode = DNSResponseCodeBadCookie
+
+func TestParseDNSBadCookie(t *testing.T) {
+	p := gopacket.NewPacket(testParseDNSBadCookie, LinkTypeNull, testDecodeOptions)
+	if p.ErrorLayer() != nil {
+		t.Error("Failed to decode packet:", p.ErrorLayer().Error())
+	}
+	checkLayers(p, []gopacket.LayerType{LayerTypeLoopback, LayerTypeIPv4, LayerTypeUDP, LayerTypeDNS}, t)
+	questions := p.Layer(LayerTypeDNS).(*DNS).Questions
+	if len(questions) != 1 {
+		t.Error("Failed to parse 1 DNS question")
+	}
+	answers := p.Layer(LayerTypeDNS).(*DNS).Answers
+	if len(answers) != 0 {
+		t.Error("Failed to parse 0 DNS answer")
+	}
+	additionals := p.Layer(LayerTypeDNS).(*DNS).Additionals
+	if len(additionals) != 1 {
+		t.Error("Failed to parse 1 DNS additional")
+	}
+
+	optAll := additionals[0].OPT
+	if len(optAll) != 1 {
+		t.Errorf("Parsed %d OPTs, expected 1", len(optAll))
+	}
+	responseCode := p.Layer(LayerTypeDNS).(*DNS).ResponseCode
+	if responseCode != testParseDNSBadCookieResponseCode {
+		t.Errorf("Incorrect extended response code, expected %q, got %q", testParseDNSBadCookieResponseCode, responseCode)
+	}
+}
+
+var testParseDNSTypeURI = []byte{
+	0x02, 0x00, 0x00, 0x00, // PF_INET
+	0x45, 0x00, 0x00, 0x6f, 0x3e, 0x65, 0x00, 0x00, 0x40, 0x11, 0x3e, 0x17, 0x7f, 0x00, 0x00, 0x01,
+	0x7f, 0x00, 0x00, 0x01, 0x00, 0x35, 0xe7, 0xd3, 0x00, 0x5b, 0xfe, 0x6e, 0xaf, 0x2d, 0x85, 0x80,
+	0x00, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01, 0x05, 0x5f, 0x68, 0x74, 0x74, 0x70, 0x03, 0x64,
+	0x6e, 0x73, 0x04, 0x74, 0x65, 0x73, 0x74, 0x00, 0x01, 0x00, 0x00, 0x01, 0xc0, 0x0c, 0x01, 0x00,
+	0x00, 0x01, 0x00, 0x00, 0x2a, 0x30, 0x00, 0x1c, 0x00, 0x0a, 0x00, 0x05, 0x68, 0x74, 0x74, 0x70,
+	0x3a, 0x2f, 0x2f, 0x77, 0x77, 0x77, 0x2e, 0x64, 0x6e, 0x73, 0x2e, 0x74, 0x65, 0x73, 0x74, 0x3a,
+	0x38, 0x30, 0x30, 0x30, 0x00, 0x00, 0x29, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+}
+var testParseDNSTypeURITarget = "http://www.dns.test:8000"
+var testParseDNSTypeURIPriority = uint16(10)
+var testParseDNSTypeURIWeight = uint16(5)
+
+func TestParseDNSTypeURI(t *testing.T) {
+	p := gopacket.NewPacket(testParseDNSTypeURI, LinkTypeNull, testDecodeOptions)
+	if p.ErrorLayer() != nil {
+		t.Error("Failed to decode packet:", p.ErrorLayer().Error())
+	}
+	checkLayers(p, []gopacket.LayerType{LayerTypeLoopback, LayerTypeIPv4, LayerTypeUDP, LayerTypeDNS}, t)
+	answers := p.Layer(LayerTypeDNS).(*DNS).Answers
+	if len(answers) != 1 {
+		t.Error("Failed to parse 1 DNS answer")
+	}
+	if len(answers[0].URI.Target) < 1 {
+		t.Error("Failed to parse 1 URI record")
+	}
+	target := string(answers[0].URI.Target)
+	if target != testParseDNSTypeURITarget {
+		t.Errorf("Incorrect URI target value, expected %q, got %q", testParseDNSTypeURITarget, target)
+	}
+	priority := answers[0].URI.Priority
+	if priority != testParseDNSTypeURIPriority {
+		t.Errorf("Incorrect URI priority value, expected %q, got %q", testParseDNSTypeURIPriority, priority)
+	}
+	weight := answers[0].URI.Weight
+	if weight != testParseDNSTypeURIWeight {
+		t.Errorf("Incorrect URI weight value, expected %q, got %q", testParseDNSTypeURIWeight, weight)
+	}
+}
+
 var testParseDNSTypeOPT = []byte{
 	0x00, 0x90, 0x0b, 0x12, 0x91, 0xc1, 0x00, 0x1c, 0xc0, 0x93, 0x33, 0xfb, 0x08, 0x00, 0x45, 0x00,
 	0x00, 0x5A, 0xce, 0x58, 0x00, 0x00, 0x40, 0x11, 0x67, 0xe2, 0xac, 0x10, 0x01, 0xc7, 0x4b, 0x4b,
@@ -215,6 +334,16 @@ func testResourceEqual(t *testing.T, i int, name string, exp, got DNSResourceRec
 	if exp.MX.Preference != got.MX.Preference {
 		t.Errorf("expected %s[%d].MX.Preference = %v, got %v", name, i, exp.MX.Preference, got.MX.Preference)
 	}
+	// URI
+	if !bytes.Equal(exp.URI.Target, got.URI.Target) {
+		t.Errorf("expected %s[%d].URI.Target = %v, got %v", name, i, exp.URI.Target, got.URI.Target)
+	}
+	if exp.URI.Weight != got.URI.Weight {
+		t.Errorf("expected %s[%d].URI.Weight = %v, got %v", name, i, exp.URI.Weight, got.URI.Weight)
+	}
+	if exp.URI.Priority != got.URI.Priority {
+		t.Errorf("expected %s[%d].URI.Priority = %v, got %v", name, i, exp.URI.Priority, got.URI.Priority)
+	}
 
 	// OPT
 	if len(exp.OPT) != len(got.OPT) {
@@ -405,6 +534,19 @@ func TestDNSEncodeResponse(t *testing.T) {
 			IP:    net.IP([]byte{1, 2, 3, 4}),
 		})
 
+	dns.Answers = append(dns.Answers,
+		DNSResourceRecord{
+			Name:  []byte("example1.com"),
+			Type:  DNSTypeURI,
+			Class: DNSClassIN,
+			TTL:   1024,
+			URI: DNSURI{
+				Target:   []byte("http://www.example2.com/"),
+				Priority: 10240,
+				Weight:   1,
+			},
+		})
+
 	dns.Answers = append(dns.Answers,
 		DNSResourceRecord{
 			Name:  []byte("www.example2.com"),
