@@ -8,6 +8,7 @@ package layers
 
 import (
 	"encoding/binary"
+	"errors"
 	"github.com/google/gopacket"
 )
 
@@ -81,27 +82,6 @@ const (
 	USBTransportTypeBulk        USBTransportType = 0x03 // Bulk transfers can be used for large bursty data, using all remaining available bandwidth, no guarantees on bandwidth or latency, such as file transfers.
 )
 
-func (a USBTransportType) LayerType() gopacket.LayerType {
-	return USBTypeMetadata[a].LayerType
-}
-
-func (a USBTransportType) String() string {
-	switch a {
-	case USBTransportTypeTransferIn:
-		return "Transfer In"
-	case USBTransportTypeIsochronous:
-		return "Isochronous"
-	case USBTransportTypeInterrupt:
-		return "Interrupt"
-	case USBTransportTypeControl:
-		return "Control"
-	case USBTransportTypeBulk:
-		return "Bulk"
-	default:
-		return "Unknown transport type"
-	}
-}
-
 type USBDirectionType uint8
 
 const (
@@ -163,6 +143,10 @@ func decodeUSB(data []byte, p gopacket.PacketBuilder) error {
 }
 
 func (m *USB) DecodeFromBytes(data []byte, df gopacket.DecodeFeedback) error {
+	if len(data) < 40 {
+		df.SetTruncated()
+		return errors.New("USB < 40 bytes")
+	}
 	m.ID = binary.LittleEndian.Uint64(data[0:8])
 	m.EventType = USBEventType(data[8])
 	m.TransferType = USBTransportType(data[9])
