@@ -156,9 +156,13 @@ func (rl *reassemblyObject) KeepFrom(offset int) {
 }
 
 func (rl *reassemblyObject) CaptureInfo(offset int) gopacket.CaptureInfo {
+	if offset < 0 {
+		return gopacket.CaptureInfo{}
+	}
+
 	current := 0
 	for _, r := range rl.all {
-		if current >= offset {
+		if current+r.length() > offset {
 			return r.captureInfo()
 		}
 		current += r.length()
@@ -280,7 +284,6 @@ type livePacket struct {
 	bytes []byte
 	start bool
 	end   bool
-	ci    gopacket.CaptureInfo
 	ac    AssemblerContext
 	seq   Sequence
 }
@@ -289,7 +292,7 @@ func (lp *livePacket) getBytes() []byte {
 	return lp.bytes
 }
 func (lp *livePacket) captureInfo() gopacket.CaptureInfo {
-	return lp.ci
+	return lp.ac.GetCaptureInfo()
 }
 func (lp *livePacket) assemblerContext() AssemblerContext {
 	return lp.ac
@@ -313,7 +316,7 @@ func (lp *livePacket) isPacket() bool {
 // Creates a page (or set of pages) from a TCP packet: returns the first and last
 // page in its doubly-linked list of new pages.
 func (lp *livePacket) convertToPages(pc *pageCache, skip int, ac AssemblerContext) (*page, *page, int) {
-	ts := lp.ci.Timestamp
+	ts := lp.captureInfo().Timestamp
 	first := pc.next(ts)
 	current := first
 	current.prev = nil
@@ -358,7 +361,7 @@ func (lp *livePacket) release(*pageCache) int {
 //    3) Call ReassemblyComplete one time, after which the stream is dereferenced by assembly.
 type Stream interface {
 	// Tell whether the TCP packet should be accepted, start could be modified to force a start even if no SYN have been seen
-	Accept(tcp *layers.TCP, ci gopacket.CaptureInfo, dir TCPFlowDirection, ackSeq Sequence, start *bool, ac AssemblerContext) bool
+	Accept(tcp *layers.TCP, ci gopacket.CaptureInfo, dir TCPFlowDirection, nextSeq Sequence, start *bool, ac AssemblerContext) bool
 
 	// ReassembledSG is called zero or more times.
 	// ScatterGather is reused after each Reassembled call,
@@ -460,6 +463,14 @@ func (c *connection) reset(k key, s Stream, ts time.Time) {
 	c.c2s.dir, c.s2c.dir = TCPDirClientToServer, TCPDirServerToClient
 }
 
+func (c *connection) lastSeen() time.Time {
+	if c.c2s.lastSeen.Before(c.s2c.lastSeen) {
+		return c.s2c.lastSeen
+	}
+
+	return c.c2s.lastSeen
+}
+
 func (c *connection) String() string {
 	return fmt.Sprintf("c2s: %s, s2c: %s", &c.c2s, &c.s2c)
 }
@@ -573,7 +584,7 @@ func NewAssembler(pool *StreamPool) *Assembler {
 	pool.users++
 	pool.mu.Unlock()
 	return &Assembler{
-		ret:              make([]byteContainer, assemblerReturnValueInitialSize),
+		ret:              make([]byteContainer, 0, assemblerReturnValueInitialSize),
 		pc:               newPageCache(),
 		connPool:         pool,
 		AssemblerOptions: DefaultAssemblerOptions,
@@ -647,7 +658,13 @@ func (a *Assembler) AssembleWithContext(netFlow gopacket.Flow, t *layers.TCP, ac
 		half.lastSeen = timestamp
 	}
 	a.start = half.nextSeq == invalidSequence && t.SYN
-	if !half.stream.Accept(t, ci, half.dir, rev.ackSeq, &a.start, ac) {
+	if *debugLog {
+		if half.nextSeq < rev.ackSeq {
+			log.Printf("Delay detected on %v, data is acked but not assembled yet (acked %v, nextSeq %v)", key, rev.ackSeq, half.nextSeq)
+		}
+	}
+
+	if !half.stream.Accept(t, ci, half.dir, half.nextSeq, &a.start, ac) {
 		if *debugLog {
 			log.Printf("Ignoring packet")
 		}
@@ -655,6 +672,9 @@ func (a *Assembler) AssembleWithContext(netFlow gopacket.Flow, t *layers.TCP, ac
 	}
 	if half.closed {
 		// this way is closed
+		if *debugLog {
+			log.Printf("%v got packet on closed half", key)
+		}
 		return
 	}
 
@@ -701,7 +721,7 @@ func (a *Assembler) AssembleWithContext(netFlow gopacket.Flow, t *layers.TCP, ac
 		}
 	}
 
-	action = a.handleBytes(bytes, seq, half, ci, t.SYN, t.RST || t.FIN, action, ac)
+	action = a.handleBytes(bytes, seq, half, t.SYN, t.RST || t.FIN, action, ac)
 	if len(a.ret) > 0 {
 		action.nextSeq = a.sendToConnection(conn, half, ac)
 	}
@@ -813,7 +833,7 @@ func (a *Assembler) checkOverlap(half *halfconnection, queue bool, ac AssemblerC
 		} else
 
 		// end < cur.end && start > cur.start: replace bytes inside cur (6)
-		if diffEnd > 0 && diffStart < 0 {
+		if diffEnd >= 0 && diffStart <= 0 {
 			if *debugLog {
 				log.Printf("case 6\n")
 			}
@@ -897,11 +917,11 @@ func (a *Assembler) dump(text string, half *halfconnection) {
 	}
 	log.Printf(" * a.ret\n")
 	for i, r := range a.ret {
-		log.Printf("\t%d: %s b:%s\n", i, r.captureInfo(), hex.EncodeToString(r.getBytes()))
+		log.Printf("\t%d: %v b:%s\n", i, r.captureInfo(), hex.EncodeToString(r.getBytes()))
 	}
 	log.Printf(" * a.cacheSG.all\n")
 	for i, r := range a.cacheSG.all {
-		log.Printf("\t%d: %s b:%s\n", i, r.captureInfo(), hex.EncodeToString(r.getBytes()))
+		log.Printf("\t%d: %v b:%s\n", i, r.captureInfo(), hex.EncodeToString(r.getBytes()))
 	}
 }
 
@@ -924,24 +944,21 @@ func (a *Assembler) overlapExisting(half *halfconnection, start, end Sequence, b
 		half.overlapPackets++
 		half.overlapBytes += diff
 	}
-	start = start.Add(diff)
 	s += diff
 	if s >= e {
 		// Completely included in sent
 		s = e
 	}
 	bytes = bytes[s:]
-	e -= diff
-	return bytes, start
+	return bytes, half.nextSeq
 }
 
 // Prepare send or queue
-func (a *Assembler) handleBytes(bytes []byte, seq Sequence, half *halfconnection, ci gopacket.CaptureInfo, start bool, end bool, action assemblerAction, ac AssemblerContext) assemblerAction {
+func (a *Assembler) handleBytes(bytes []byte, seq Sequence, half *halfconnection, start bool, end bool, action assemblerAction, ac AssemblerContext) assemblerAction {
 	a.cacheLP.bytes = bytes
 	a.cacheLP.start = start
 	a.cacheLP.end = end
 	a.cacheLP.seq = seq
-	a.cacheLP.ci = ci
 	a.cacheLP.ac = ac
 
 	if action.queue {
@@ -1053,7 +1070,16 @@ func (a *Assembler) cleanSG(half *halfconnection, ac AssemblerContext) {
 	half.saved = nil
 	var saved *page
 	for _, r := range a.cacheSG.all[ndx:] {
+		preConvertLen := r.length()
 		first, last, nb := r.convertToPages(a.pc, skip, ac)
+
+		// Update skip count as we move from one container to the next.
+		if delta := preConvertLen - r.length(); delta > skip {
+			skip = 0
+		} else {
+			skip -= delta
+		}
+
 		if half.saved == nil {
 			half.saved = first
 		} else {
@@ -1273,9 +1299,11 @@ func (a *Assembler) flushClose(conn *connection, half *halfconnection, t time.Ti
 		a.skipFlush(conn, half)
 		if half.closed {
 			closed = true
+			return flushed, closed
 		}
 	}
-	if !half.closed && half.first == nil && half.lastSeen.Before(tc) {
+	// Close the connection only if both halfs of the connection last seen before tc.
+	if !half.closed && half.first == nil && conn.lastSeen().Before(tc) {
 		a.closeHalfConnection(conn, half)
 		closed = true
 	}
